import scipy.stats as stats
from scipy.spatial.distance import pdist, squareform
import numpy as np
import math
import pickle
import itertools
import logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
handler = logging.FileHandler("weeks.log")
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)


count = 0
total = 0
def create_similarity_matrix(features, sim_func, key_filepath,inverse_freqs = None, significance_weight = 0, default_voting = True):
    keys = sorted(list(features.keys()))
    keys = np.asarray(keys, dtype=int)
    n = keys.shape[0]
    keys.shape = (n,1)
    pickle.dump(keys, open(key_filepath, "wb"))
    print(keys.shape)
    global count
    global total
    total = (n*(n-1))/2
    count = 0


    def custom_dist_fwpc(x,y):
        global total
        global count
        count += 1
        if count % 100000 == 0:
            logger.info("Computed " + str(count) + " similarities out of a total of " + str(total)+"." + str(total-count)
                        + " calculations remaining")
        this, other, weights = [], [], []
        x = int(x[0])
        y = int(y[0])
        this_dict, other_dict = features[x], features[y]
        this_keys, other_keys = set(this_dict.keys()), set(other_dict.keys())
        rated_by_both = this_keys.intersection(other_keys)
        if len(rated_by_both) == 0:
            sim = 0
        else:
            if default_voting:
                rated_by_both = other_keys.union(this_keys)
            for key in rated_by_both:
                this.append(float(this_dict.get(key, 0)))
                other.append(float(other_dict.get(key, 0)))
                weights.append(inverse_frequency_dict[key])
            sim = fwpearsonr(this, other, weights)

            if sim < 0 or math.isnan(sim):
                sim = 0
            if significance_weight:
                sim *= min(significance_weight, len(rated_by_both)) / float(significance_weight)
                # if sim < 0.3:
                #     sim = 0
        return sim

    def custom_dist(x,y):
        global total
        global count
        count +=1
        if count % 100000 == 0:
            logger.info(
                "Computed " + str(count) + " similarities out of a total of " + str(total) + ".\n" + str(total - count)
                + " calculations remaining")
        this, other = [], []
        x = int(x[0])
        y = int(y[0])
        this_dict, other_dict = features[x], features[y]
        this_keys, other_keys = set(this_dict.keys()), set(other_dict.keys())
        rated_by_both = this_keys.intersection(other_keys)
        if len(rated_by_both) == 0:
            sim = 0
        else:
            if default_voting and sim_func == stats.pearsonr:
                rated_by_both = other_keys.union(this_keys)
            for key in rated_by_both:
                this.append(float(this_dict.get(key, 0)))
                other.append(float(other_dict.get(key, 0)))
            sim, rho = sim_func(this, other)

            if sim < 0 or math.isnan(sim):
                sim = 0
                if math.isnan(sim):
                    print(this,other)
            if significance_weight:
                sim *= min(significance_weight, len(rated_by_both)) / float(significance_weight)
                # if sim < 0.3:
                #     sim = 0
        return sim

    if sim_func == fwpearsonr:
        #inverse_frequency_dict = pickle.load(
         #   open("/home/sondredyvik/masteroppgave/results/fwpc/inverse_frequencies.pickle", "rb"))
        if inverse_freqs == None:
            inverse_frequency_dict = pickle.load(
                open(r"/home/sondredyvik/masteroppgave/results/fwpc/inverse_frequencies.pickle", "rb"))
        else:
            inverse_frequency_dict = inverse_freqs

        return squareform(pdist(keys, metric=custom_dist_fwpc))
    else:
        return squareform(pdist(keys, metric=custom_dist))

def fwpearsonr(x, y, fw):
    """
    Calculates a Pearson correlation coefficient and the p-value for testing
    non-correlation.

    The Pearson correlation coefficient measures the linear relationship
    between two datasets. Strictly speaking, Pearson's correlation requires
    that each dataset be normally distributed, and not necessarily zero-mean.
    Like other correlation coefficients, this one varies between -1 and +1
    with 0 implying no correlation. Correlations of -1 or +1 imply an exact
    linear relationship. Positive correlations imply that as x increases, so
    does y. Negative correlations imply that as x increases, y decreases.

    The p-value roughly indicates the probability of an uncorrelated system
    producing datasets that have a Pearson correlation at least as extreme
    as the one computed from these datasets. The p-values are not entirely
    reliable but are probably reasonable for datasets larger than 500 or so.

    Parameters
    ----------
    x : (N,) array_like
        Input
    y : (N,) array_like
        Input

    Returns
    -------
    r : float
        Pearson's correlation coefficient
    p-value : float
        2-tailed p-value

    References
    ----------
    http://www.statsoft.com/textbook/glosp.html#Pearson%20Correlation

    """
    # x and y should have same length.
    x = np.asarray(x)
    y = np.asarray(y)
    fw = np.asarray(fw)
    n = len(x)
    mx = x.mean()
    my = y.mean()
    xm, ym = x - mx, y - my
    r_num = np.add.reduce(xm * ym * fw)
    r_den = np.sqrt(np.sum(fw*np.square(xm)) * np.sum(fw*np.square(xm)) )
    r = r_num / r_den

    # Presumably, if abs(r) > 1, then it is only some small artifact of floating
    # point arithmetic.
    r = max(min(r, 1.0), -1.0)
    return r

def sum_of_squares(a, axis=0):
    """
    Squares each element of the input array, and returns the sum(s) of that.

    Parameters
    ----------
    a : array_like
        Input array.
    axis : int or None, optional
        Axis along which to calculate. Default is 0. If None, compute over
        the whole array `a`.

    Returns
    -------
    sum_of_squares : ndarray
        The sum along the given axis for (a**2).

    See also
    --------
    _square_of_sums : The square(s) of the sum(s) (the opposite of
    `_sum_of_squares`).
    """
    a, axis = _chk_asarray(a, axis)
    return np.sum(a*a, axis)


def _chk_asarray(a, axis):
    if axis is None:
        a = np.ravel(a)
        outaxis = 0
    else:
        a = np.asarray(a)
        outaxis = axis

    if a.ndim == 0:
        a = np.atleast_1d(a)

    return a, outaxis