import community
import networkx as nx
import numpy as np

from generate_similarity_matrix import create_similarity_matrix, fwpearsonr

from collections import defaultdict
# import math
import pickle
import scipy.stats as stats
# from sklearn.cluster import DBSCAN, SpectralClustering, SpectralBiclustering, SpectralCoclustering,AffinityPropagation
import matplotlib.pyplot as plt
# import pyclust
import copy
from queries import SQL_GET_EMNE, SQL_GET_FILDOKUMENT, SQL_GET_KONTOPLAN
# import itertools
# from cluster_k_medoid import kMedoids
from sklearn.metrics import silhouette_score
import time
import random
import datetime
from analysis_script import write_trending_categories_for_clusters_month
# import pyodbc
import joblib
import os
import calendar
import math
import pyodbc
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
handler = logging.FileHandler("connections_and_clustering.log")
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)


def create_id_to_index(user_set):
    index_list = list(user_set)
    index_list.sort()
    id_to_index_dict = dict()
    index_to_id_dict = dict()
    for user_id in index_list:
        id_to_index_dict[user_id] = index_list.index(user_id)
        index_to_id_dict[index_list.index(user_id)] = user_id
    print('Id to index dict created')
    return id_to_index_dict, index_to_id_dict



def write_matrix_to_graph_file(matrix,filename,index_to_id):
    with open(filename, "w") as handle:
        handle.write(";")
        [handle.write(str(index_to_id[i]) + ";") for i in range(matrix.shape[0])]
        for j in range(matrix.shape[0]):
            handle.write("\n")
            handle.write(str(index_to_id[j])+";")
            slice = matrix[:, j:j+1]
            [handle.write(str(float(element[0]))+";") for element in slice]
    print("written")

def plot_distance_to_k_closest(connections, k):
    dist = []
    for row in connections:
        dist.append(sorted(row)[k])
    plt.plot(sorted(dist))
    plt.title(str(k))
    plt.show()


def compute_inverse_frequencies(hit_dict):
    subjects_total_votes = dict()
    for key in hit_dict.keys():
        for sub_key in hit_dict[key]:
            if sub_key in subjects_total_votes.keys():
                subjects_total_votes[sub_key] += 1
            else:
                subjects_total_votes[sub_key] = 1
    total = len(list(hit_dict.keys()))
    for key in subjects_total_votes:
        subjects_total_votes[key] = np.log(total/subjects_total_votes[key])
    return subjects_total_votes

def find_most_common_topic_of_user(user_id, hit_dict):
    tuplist = []
    for key in hit_dict[user_id]:
        tuplist.append((key, hit_dict[user_id][key]))
    tuplist = sorted(tuplist, key = lambda x: x[1], reverse = True)
    return tuplist[:10]


def find_most_common_topics_of_cluster(user_ids, hit_dict):
    topics = dict()
    for user in user_ids:
        results = find_most_common_topic_of_user(user, hit_dict)
        for i in range(len(results)):
            if results[i][0] in topics.keys():
                topics[results[i][0]] += 1
            else:
                topics[results[i][0]] = 1
    # print(sorted(list (map(lambda x: x/len(user_ids), topics.values()))))

    # topics = zip(topics.keys(), list (map(lambda x: x/len(user_ids), topics.values())))
    topics = zip(topics.keys(), topics.values())
    topics = sorted(topics, key= lambda x: x[1],reverse= True)[:20]
    return topics

# def find_most_common_topics_by_cluster(cluster_dict, hit_dict,algorithm):
#     connection_in = pyodbc.connect(r'DSN=studentdsn; Trusted_Connection=yes;')
#     cursor = connection_in.cursor()
#     with open(algorithm +"_common_clusters.txt", "wb") as handle:
#         out = ""
#         for cluster in sorted(list(cluster_dict.keys())):
#             results = find_most_common_topics_of_cluster(cluster_dict[cluster], hit_dict)
#             out += "Cluster Number: " + str(cluster) + "|  Members in this cluster: " + str(len(cluster_dict[cluster])) + "\n"
#
#             for result in results:
#                 resource = result[0]
#                 if resource[1] == 1:
#                     command = SQL_GET_EMNE.format(resource[0])
#                 if resource[1] == 2:
#                     command = SQL_GET_FILDOKUMENT.format(resource[0])
#                 if resource[1] == 3:
#                     command = SQL_GET_KONTOPLAN.format(resource[0])
#                 cursor.execute(command)
#                 title = cursor.fetchone()
#
#                 if title == None:
#                     title =resource
#                 else:
#                     title = title[0]
#
#                 out +=str(title).ljust(80) + ": " + str(result[1]) + "\n"
#             out += '\n'
#         handle.write(out.encode('utf8'))

def create_clustering_for_month(month_string = 'April2015'):
    """CLUSTERING"""
    logger.info('Importing data for '+ month_string)
    month = month_string
    data = pickle.load( open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\connections-"+ month +".pickle", "rb"))
    logger.info('Data imported successfully, converting data to graph')
    G = nx.from_numpy_matrix(data)
    logger.info('Data converted successfully, creating best partition')
    partition = community.best_partition(G)
    logger.info('Best partition created successfully, loading keys')
    keys = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\keys-"+ month +".pickle","rb"))
    size = float(len(set(partition.values())))
    labels = list(partition.values())
    index_to_key = dict()
    cluster_dict_with_id = dict()
    for i in range(len(keys)):
        index_to_key[i] = keys[i]
        cluster = labels[i]
        key = int(keys[i])
        if cluster in cluster_dict_with_id:
            cluster_dict_with_id[cluster].add(key)
        else:
            cluster_dict_with_id[cluster] = set()
            cluster_dict_with_id[cluster].add(key)

    del_keys = list()
    for key in cluster_dict_with_id.keys():
        if len(cluster_dict_with_id[key])< 3:
            del_keys.append(key)

    for key in del_keys:
        cluster_dict_with_id.pop(key)

    pickle.dump(cluster_dict_with_id, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\clusters-" + month + ".pickle","wb"))

def transform_from_distance_to_similarity(connections):
    f = lambda x: (1-x )
    f = np.vectorize(f)
    return f(connections)


def print_closest_users(connections,index_to_id):
    closest_user = dict()
    index = 0
    for row in connections:
        index_of_max = np.argmax(row)
        closest_user[index_to_id[index]] = (index_to_id[index_of_max], row[index_of_max])
        index += 1

    pickle.dump(closest_user, open(r"C:\Users\Student\PycharmProjects\masteroppgave\pickles\nearest_neighbor_dict.pickle","wb"))
    d = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\pickles\nearest_neighbor_dict.pickle","rb"))
    p = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\pickles\id_to_position.pickle","rb"))

    keys = list(d.keys())

    data_source = set(p.keys()).intersection(set(keys))
    other_source = set()
    for key in data_source:
        other_source.add(d[key][0])


    other_source = other_source.intersection(set(p.keys()))
    with open("spearmanr-closest-users","wb") as handle:
        for key in data_source:
            if d[key][0] in other_source:
                out = str(p[key]) + " == " +str(d[key][1]) + " ==> "+ str(p[d[key][0]])+"\n"
                handle.write(out.encode('utf8'))

prog_start = time.time()

# # #
# #

# #
# pickle.dump(subjects_total_votes, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\inverse_frequencies.pickle","wb"))

# #
# pickle.dump(normalized_dict, open(r"C:\Users\Student\PycharmProjects\masteroppgave\pickles\normalized_dict.pickle","wb"))


# with open("statistics.txt", "w") as handle:
#     handle.write("Matrix generation statistics\n")
# fwpc = False
# spearmanr = False
# pearsonr = False
# users = False
# weeks = False


# fwpc, spearmanr, pearsonr, users, weeks = True, True, True, False, True
fwpc, spearmanr, pearsonr, users, weeks, months = False, False, False, False, False,False


def create_connections_for_users(fwpc = False, spearmanr =False , pearsonr = False, user_significance_weight=10, size_of_selection_users=10000):
    hit_dict = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\hit_dict_index_on_user_id.pickle","rb"))
    # hit_dict = pickle.load(open(r"C:\Users\Student\PycharmProjects\maseroppgave\data\hit_dict_index_on_user_id.pickle","rb"))
    normalized_dict = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\normalized_hit_dict_index_on_user_id.pickle","rb"))
    size_of_selection = size_of_selection_users
    for key in hit_dict.keys():
        if len(list(hit_dict[key].keys())) < 20 and sum(hit_dict[key].values()) < 100:
            normalized_dict.pop(key)
    print(len(set(normalized_dict.keys())))
    allkeys = copy.deepcopy(list(normalized_dict.keys()))
    random.shuffle(allkeys)
    keep = allkeys[:size_of_selection]
    discard = allkeys[size_of_selection:]
    for key in discard:
        normalized_dict.pop(key)


    if fwpc:
        start = time.time()
        key_filepath = r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\users\keys.pickle"
        distances = create_similarity_matrix(normalized_dict, fwpearsonr, key_filepath, significance_weight=user_significance_weight, default_voting= True)
        pickle.dump(distances, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\users\random"+ str(size_of_selection)+".pickle", "wb"))
        secs = time.time() - start
        print("user fwpc", size_of_selection,str(datetime.timedelta(seconds=secs)) )
        with open("statistics.txt", "a") as handle:
            handle.write("FWPC for "+str(size_of_selection)+" users created in " + str(datetime.timedelta(seconds=secs))+ " seconds\n" )
    if spearmanr:
        start = time.time()
        key_filepath = r"C:\Users\Student\PycharmProjects\masteroppgave\results\src\users\keys.pickle"
        distances = create_similarity_matrix(normalized_dict, stats.spearmanr, key_filepath, significance_weight=user_significance_weight, default_voting= True)
        pickle.dump(distances, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\src\users\random"+ str(size_of_selection)+".pickle", "wb"))
        secs = time.time() - start
        print("user spearman", size_of_selection, str(datetime.timedelta(seconds=secs)))
        with open("statistics.txt", "a") as handle:
            handle.write("Spearman rank for "+str(size_of_selection)+" users created in " + str(datetime.timedelta(seconds=secs))+ " seconds\n" )
    if pearsonr:
        start = time.time()
        key_filepath = r"C:\Users\Student\PycharmProjects\masteroppgave\results\pearson\users\keys.pickle"
        distances = create_similarity_matrix(normalized_dict, stats.pearsonr, key_filepath, significance_weight=user_significance_weight, default_voting= True)
        pickle.dump(distances, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\pearson\users\random"+ str(size_of_selection)+".pickle", "wb"))
        secs = time.time() - start
        print("user pearson", size_of_selection, str(datetime.timedelta(seconds=secs)))
        with open("statistics.txt", "a") as handle:
            handle.write("Pearson rank for "+str(size_of_selection)+" users created in " + str(datetime.timedelta(seconds=secs))+ " seconds\n" )


def create_connections_for_all_months(default_voting = True, significance_weight = 4):
    data = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\data_by_month\normalized_all_months","rb"))
    # month_name_to_date_time = pickle.load(
    #     open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\data_by_month\month_name_to_date_time.pickle", "rb"))
    for key in list(data.keys()):
        intermediate = time.time()
        month = data[key]
        filename = str(calendar.month_name[key[0].month]) + str(key[0].year)
        try:
            inverse_freqs = compute_inverse_frequencies(month)
            print(filename)
            key_filepath = str(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\keys-"+filename+".pickle")
            print(len(list(month.keys())))
            logger.debug('Creating connections for ' + filename + "\n")
            connections = create_similarity_matrix(month, fwpearsonr, key_filepath, inverse_freqs,  significance_weight, default_voting)
            secs = time.time() - intermediate
            logger.debug('Connections created in ' + str(
                datetime.timedelta(seconds=secs)) + " seconds\n")
        except Exception as e:
            logger.error(filename+ "not created", exc_info = True)
        else:
            try:
                pickle.dump(connections, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\connections-"+filename+".pickle", "wb"))
            except Exception as b:
                logger.error("Could not pickle" + filename)
        secs = time.time() - intermediate
        with open("statistics.txt", "a") as handle:
            handle.write("Connections for " + filename+ "created in" + str(
                datetime.timedelta(seconds=secs)) + " seconds\n")
        create_clustering_for_month(filename)
        secs = time.time() - secs
        with open("statistics.txt", "a") as handle:
            handle.write("Clusters for " + filename+ "created in" + str(
                datetime.timedelta(seconds=secs)) + " seconds\n")
        write_trending_categories_for_clusters_month(filename, month_name_to_date_time )

    with open("statistics.txt", "a") as handle:
        handle.write("Data for all months created in " + str(
            datetime.timedelta(seconds=secs)) + " seconds\n")


# # # # # # #
# print("loading data...")
# print("jeg")
#



# index_to_id = dict()
# print("loading keys...")
# keys = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\src\users\keys.pickle","rb"))
# for i in range(len(keys)):
#     index_to_id[i] = keys[i][0]
# # #
# similarities = transform_from_distance_to_similarity(distances)

# model = SpectralClustering(affinity='precomputed').fit(data)
# labels = model.labels_
# model = DBSCAN(eps=0.3, min_samples=5, metric='precomputed').fit(data)

# c,m = kMedoids(distances,   5)
# for key in m.keys():
#     print(key, len(list(m[key])))


# # labels =[]
# # for i in range(similarities.shape[1]):
# #     for key in m.keys():
# #         if i in m[key]:
# #             labels.append(key)
#
#
#
# labels = model.labels_
# print(set(labels))
#

# #
# # for row in similarities:
# # #     print(sorted(row, reverse=True)[:10])

# dendrogram = community.generate_dendrogram(G)
# for level in range(len(dendrogram) - 1) :
#     print("partition at level", level, "is", community.partition_at_level(dendrogram, level))
# #
# # #

#
# # #

def create_clustering_year(connections_path, key_path, save_path):
    data = pickle.load(open(connections_path,"rb"))
    G = nx.from_numpy_matrix(data)
    partition = community.best_partition(G)
    keys = pickle.load(open(key_path,"rb"))
    print(keys)
    labels = list(partition.values())
    index_to_key = dict()
    cluster_dict_with_id = dict()
    for i in range(len(keys)):
        index_to_key[i] = keys[i]
        cluster = labels[i]
        key = int(keys[i])
        if cluster in cluster_dict_with_id:
            cluster_dict_with_id[cluster].add(key)
        else:
            cluster_dict_with_id[cluster] = set()
            cluster_dict_with_id[cluster].add(key)

    del_keys = list()
    for key in cluster_dict_with_id.keys():
        if len(cluster_dict_with_id[key])< 3:
            del_keys.append(key)

    for key in del_keys:
        cluster_dict_with_id.pop(key)
    #
    pickle.dump(cluster_dict_with_id, open(save_path,"wb"))

#
# for i in range(1,13):
#     for year in ['2015','2016','2017']:
#         month = calendar.month_name[i]+year
#         try:
#             cluster_dict_with_id = pickle.load(open(
#                 r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\clusters-" + month + ".pickle",
#                 "rb"))
#             print(month)
#             for key in cluster_dict_with_id.keys():
#
#                 print(key, len(list(cluster_dict_with_id[key])))
#             print("============================================")
#             # create_clustering_for_month(month)
#         except FileNotFoundError as E:
#             pass
#             # logger.error(month + "does not exist", exc_info =True)



# month = 'April2016'
# create_clustering_for_month(month)
"""Clustering end"""
# pickle.dump(cluster_dict_with_id, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\src\users\clusters_with_id.pickle","wb"))

#
# SQL_emne_fagomrade = """SELECT Emner, Fagomrader
#             FROM [RegSticosOppslag].[dbo].[EmneFagomrader]"""
# SQL_fagomrade = """
# SELECT Navn
# FROM [RegSticosOppslag].[dbo].[Fagomrade]
# WHERE Id = {0}
#  """
#

# emne_fagomrade_dict = dict()
# cursor = connection_in.cursor()
# fagomrade_cursor = fagomrade_conn.cursor()
# cursor.execute(SQL_emne_fagomrade)
# results = cursor.fetchone()
# while results:
#     emne, fagomradeid = results
#     fagomrade_cursor.execute(SQL_fagomrade.format(fagomradeid))
#     navn = fagomrade_cursor.fetchone()[0]
#     emne_fagomrade_dict[emne] = navn
#     print(navn)
#     results = cursor.fetchone()
# pickle.dump(emne_fagomrade_dict,open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\emne_to_fagomrade_navn.pickle","wb") )
# print(emne_fagomrade_dict)

#
# for key in emne_to_category.keys():
#     categories = emne_to_category[key]
#     emne_to_category_name[key] = list()
#     for category in categories:
#         print(category)
#         cursor.execute(SQL_fagomrade.format(category))
#         name = cursor.fetchone()[0]
#         emne_to_category_name[key].append(name)
# pickle.dump(emne_to_category_name,open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\emne_to_category_name.pickle"))
def write_trending_categories_for_clusters_month_averages_subtracted(month):
    data = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\data_by_month\normalized_all_months", "rb"))
    clusters_with_id = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\clusters-" + month + ".pickle", "rb"))
    metadata = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\usermetadata.pickle", "rb"))
    month_name_to_date_time = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\data_by_month\month_name_to_date_time.pickle", "rb"))
    metadata = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\usermetadata.pickle", "rb"))
    data = data[month_name_to_date_time[month]]
    emne_to_fagomrade_navn = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\emne_to_category_name.pickle", "rb"))
    emner = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\emner.pickle", "rb"))
    kontoplaner = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\kontoplaner.pickle", "rb"))
    fildokumenter = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\fildokumenter.pickle", "rb"))

    top_list_all_clusters = dict()
    top_list_of_resources_all_clusters = dict()
    all_ratings_dict = defaultdict(list)
    total_users = 0

    for cluster_key in clusters_with_id.keys():
        users = clusters_with_id[cluster_key]
        for user_id in users:
            total_users +=1
            user_dict = data[user_id]
            for key in user_dict.keys():
                all_ratings_dict[key].append(user_dict[key])
    all_ratings_dict = {key: np.sum(values)/total_users for key, values in all_ratings_dict.items()}

    for cluster_key in clusters_with_id.keys():
        users = clusters_with_id[cluster_key]
        cluster_top_dict = dict()
        cluster_resource_top_dict = defaultdict(list)
        cluster_total_users = 0
        for user_id in users:
            user_dict = data[user_id]
            cluster_total_users +=1
            tuplist = []
            resource_tuplist = []
            for key in user_dict:
                value = user_dict[key]
                cluster_resource_top_dict[key] = value

        cluster_resource_top_dict = {key: (np.sum(values)/len(users))- all_ratings_dict[key] for key, values in cluster_resource_top_dict.items()}

        cluster_resource_top_dict_with_names = dict()
        for key in cluster_resource_top_dict.keys():
            if key[1] == 1:
                new_key = "Emne: " + emner.get(key[0], 'slettet')
                cluster_resource_top_dict_with_names[new_key] = (cluster_resource_top_dict[key], all_ratings_dict[key])

            if key[1] == 3:
                new_key = "Kontoplan: " + kontoplaner[key[0]]
                cluster_resource_top_dict_with_names[new_key] = (cluster_resource_top_dict[key], all_ratings_dict[key])
            if key[1] == 2:
                new_key = "Fildokument: " + fildokumenter.get(key[0], 'slettet')
                cluster_resource_top_dict_with_names[new_key] = (cluster_resource_top_dict[key], all_ratings_dict[key])
        cluster_resource_top_dict = cluster_resource_top_dict_with_names

        top_list = sorted(list(zip(cluster_top_dict.keys(), cluster_top_dict.values())), key=lambda x: x[1],
                          reverse=True)
        top_list_of_resources = sorted(
            list(zip(cluster_resource_top_dict.keys(), cluster_resource_top_dict.values())), key=lambda x: x[1][0],
            reverse=True)

        # max_resource_top_list = max(top_list_of_resources, key=lambda x: x[1])[1]
        # max_top_list = max(top_list, key=lambda x: x[1])[1]
        # top_list = list(map(lambda x: (x[0], x[1] / max_top_list), top_list))
        # top_list_of_resources = list(
        #     list(map(lambda x: (x[0], x[1] / max_resource_top_list), top_list_of_resources)))
        top_list_of_resources_all_clusters[cluster_key] = top_list_of_resources

        top_list_all_clusters[cluster_key] = top_list
    cluster_data = dict()
    for cluster_key in clusters_with_id.keys():
        cluster_user_data = dict()
        cluster = clusters_with_id[cluster_key]
        total_users_with_data = 0
        total_users = 0
        for user in cluster:
            total_users += 1
            if user in metadata.keys():
                total_users_with_data += 1
                userdict = metadata[user]
                for data_key in userdict.keys():
                    if userdict[data_key] != "":
                        if data_key in cluster_user_data.keys():
                            if type(userdict[data_key]) == list:
                                cluster_user_data[data_key].extend(userdict[data_key])
                            else:
                                cluster_user_data[data_key].append(userdict[data_key])
                        else:
                            if type(userdict[data_key]) == list:
                                cluster_user_data[data_key] = list(userdict[data_key])
                            else:
                                cluster_user_data[data_key] = [userdict[data_key]]

        count_dict = dict()
        for key in cluster_user_data.keys():
            vals = set(cluster_user_data[key])
            if key.lower() == "bransje": vals = ["Regnskapsbyrå", "Revisjonsselskap", "Other"]
            temp_dict = dict()
            for val in vals:
                if val != "Other":
                    temp_dict[val] = cluster_user_data[key].count(val)
                else:
                    regnskap = cluster_user_data[key].count("Regnskapsbyrå")
                    revisjon = cluster_user_data[key].count("Revisjonsselskap")
                    temp_dict[val] = len(cluster_user_data[key]) - regnskap - revisjon
            for value_key in temp_dict.keys():
                # temp_dict[value_key] = math.ceil((temp_dict[value_key]/total)*100)
                temp_dict[value_key] = '{0:.3g}'.format((temp_dict[value_key] / total_users_with_data) * 100)
            count_dict[key] = temp_dict
        cluster_data[cluster_key] = (count_dict, total_users_with_data, total_users)

    with open("clusteranalysis.txt", "wb") as handle:
        count = 0
        out = ""
        for cluster in clusters_with_id.keys():
            members = len(clusters_with_id[cluster])
            count += 1
            out += "Cluster number " + str(count) + ":  \nNumber of users in this cluster: " + str(members) + "\n"
            out += "============================================\n"
            top_list = top_list_all_clusters[cluster]
            resource_top_list = top_list_of_resources_all_clusters[cluster]
            print(max(top_list_of_resources_all_clusters[cluster], key=lambda x: x[1][0]))
            for val in top_list[:10]:
                out += str(val[0]) + ";" + str(val[1]) + "\n"
            out += "\n \n \n Resources: \n"
            for resource_val in resource_top_list[:20]:
                out += str(resource_val[0]) + ";" + str(resource_val[1][0] + resource_val[1][1])+";" +str(resource_val[1][1]) + "\n"
            out += "\n \n \n Medians: \n"

            data_dict, num_meta, num_total = cluster_data[cluster]

            out += "Metadata: \n"
            out += "Number of users for which we have metadata: " + str(num_meta) + "\n"
            for key in ["stilling", "bransje"]:
                out += str(key).upper() + ":\n"
                try:
                    sorted_values = sorted(list(zip(data_dict[key].keys(), data_dict[key].values())),
                                           key=lambda x: float(x[1]), reverse=True)
                except KeyError as e:
                    if num_meta == 0:
                        logger.error("No metadata")

                for val in sorted_values:
                    out += str(val[0]) + ";" + str(val[1]) + "\n"
            out += "\n \n \n"
        handle.write(out.encode("utf8"))



def write_trending_categories_for_clusters_year(cluster_path, data_path):
    data = pickle.load(
        open(data_path, "rb"))
    clusters_with_id = pickle.load(
        open(cluster_path,
             "rb"))
    metadata = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\usermetadata.pickle", "rb"))
    emne_to_fagomrade_navn = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\emne_to_category_name.pickle", "rb"))
    emner = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\emner.pickle", "rb"))
    kontoplaner = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\kontoplaner.pickle", "rb"))
    fildokumenter = pickle.load(
        open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\fildokumenter.pickle", "rb"))

    top_list_all_clusters = dict()
    top_list_of_resources_all_clusters = dict()
    key_translator = dict()
    all_ratings_dict = defaultdict(list)
    for cluster_key in clusters_with_id.keys():
        users = clusters_with_id[cluster_key]
        for user_id in users:
            user_dict = data[user_id]
            for key in user_dict.keys():
                all_ratings_dict[key] = user_dict[key]
    all_ratings_dict = {key: np.average(values) for key, values in all_ratings_dict.items()}

    for cluster_key in clusters_with_id.keys():
        users = clusters_with_id[cluster_key]
        cluster_top_dict = dict()
        cluster_resource_top_dict = defaultdict(list)

        for user_id in users:
            user_dict = data[user_id]
            tuplist = []
            resource_tuplist = []
            for key in user_dict:
                value = user_dict[key]
                if key[0] in emne_to_fagomrade_navn.keys() and key[1] == 1:
                    if type(emne_to_fagomrade_navn[key[0]]) == set:
                        list_of_categories = emne_to_fagomrade_navn[key[0]]
                        for category in list_of_categories:
                            tuplist.append((category, value))
                    else:
                        tuplist.append((emne_to_fagomrade_navn[key[0]],value))
                        if key[0] in emner.keys() and key[1] == 1:
                            resource_tuplist.append(('Emne: ' + emner[key[0]],value))

                        else:
                            print(key[0] in emner.keys())
                            resource_tuplist.append(('slettet', value))

                if key[1] == 3:
                    tuplist.append(('Kontoplan', value))
                    resource_tuplist.append(("Kontoplan: " + kontoplaner[key[0]],value))
                    #

                if key[1] == 2:
                    if key[0] in fildokumenter.keys():
                        tuplist.append(('Fildokument',value))
                        resource_tuplist.append(('Fildokument: ' + fildokumenter[key[0]],value))

                    else:
                        resource_tuplist.append(('slettet fildokument:', value))

            try:
                max_val = max(tuplist, key=lambda x: x[1])[1]
                max_resource = max(resource_tuplist, key=lambda x: x[1])[1]

                resource_tuplist = list(map(lambda x: (x[0], x[1] / max_resource), resource_tuplist))
                tuplist = list(map(lambda x: (x[0], x[1] / max_val), tuplist))
                tuplist = sorted(tuplist, key=lambda x: x[1], reverse=True)
                resource_tuplist = sorted(resource_tuplist, key=lambda x: x[1], reverse=True)


            except ValueError as e:
                pass
            for tup in tuplist:
                if tup[0] in cluster_top_dict.keys():
                    cluster_top_dict[tup[0]] += tup[1]
                else:
                    cluster_top_dict[tup[0]] = tup[1]
            for tup in resource_tuplist:
                if tup[0] in cluster_resource_top_dict.keys():
                    cluster_resource_top_dict[tup[0]] += tup[1]
                else:
                    cluster_resource_top_dict[tup[0]] = tup[1]




        top_list = sorted(list(zip(cluster_top_dict.keys(), cluster_top_dict.values())), key=lambda x: x[1],
                          reverse=True)
        top_list_of_resources = sorted(
            list(zip(cluster_resource_top_dict.keys(), cluster_resource_top_dict.values())), key=lambda x: x[1],
            reverse=True)
        max_resource_top_list = max(top_list_of_resources, key=lambda x: x[1])[1]
        max_top_list = max(top_list, key=lambda x: x[1])[1]
        top_list = list(map(lambda x: (x[0], x[1] / max_top_list), top_list))
        top_list_of_resources = list(
            list(map(lambda x: (x[0], x[1] / max_resource_top_list), top_list_of_resources)))
        top_list_of_resources_all_clusters[cluster_key] = top_list_of_resources
        top_list_all_clusters[cluster_key] = top_list
    cluster_data = dict()
    for cluster_key in clusters_with_id.keys():
        cluster_user_data = dict()
        cluster = clusters_with_id[cluster_key]
        total_users_with_data = 0
        total_users = 0
        for user in cluster:
            total_users += 1
            if user in metadata.keys():
                total_users_with_data += 1
                userdict = metadata[user]
                for data_key in userdict.keys():
                    if userdict[data_key] != "":
                        if data_key in cluster_user_data.keys():
                            if type(userdict[data_key]) == list:
                                cluster_user_data[data_key].extend(userdict[data_key])
                            else:
                                cluster_user_data[data_key].append(userdict[data_key])
                        else:
                            if type(userdict[data_key]) == list:
                                cluster_user_data[data_key] = list(userdict[data_key])
                            else:
                                cluster_user_data[data_key] = [userdict[data_key]]

        count_dict = dict()
        for key in cluster_user_data.keys():
            vals = set(cluster_user_data[key])
            if key.lower() == "bransje": vals = ["Regnskapsbyrå", "Revisjonsselskap", "Other"]
            temp_dict = dict()
            for val in vals:
                if val != "Other":
                    temp_dict[val] = cluster_user_data[key].count(val)
                else:
                    regnskap = cluster_user_data[key].count("Regnskapsbyrå")
                    revisjon = cluster_user_data[key].count("Revisjonsselskap")
                    temp_dict[val] = len(cluster_user_data[key]) - regnskap - revisjon
            for value_key in temp_dict.keys():
                # temp_dict[value_key] = math.ceil((temp_dict[value_key]/total)*100)
                temp_dict[value_key] = '{0:.3g}'.format((temp_dict[value_key] / total_users_with_data) * 100)
            count_dict[key] = temp_dict
        cluster_data[cluster_key] = (count_dict, total_users_with_data, total_users)

    with open("clusteranalysis.txt", "wb") as handle:
        count = 0
        out = ""
        for cluster in clusters_with_id.keys():
            members = len(clusters_with_id[cluster])
            count += 1
            out += "Cluster number " + str(count) + ":  \nNumber of users in this cluster: " + str(members) + "\n"
            out += "============================================\n"
            top_list = top_list_all_clusters[cluster]
            resource_top_list = top_list_of_resources_all_clusters[cluster]

            for val in top_list[:10]:
                out += str(val[0]) + ";" + str(val[1]) + "\n"
            out += "\n \n \n Resources: \n"
            for resource_val in resource_top_list[:20]:
                out += str(resource_val[0]) + ";" + str(resource_val[1]) + "\n"
            out += "\n \n \n Medians: \n"

            data_dict, num_meta, num_total = cluster_data[cluster]

            out += "Metadata: \n"
            out += "Number of users for which we have metadata: " + str(num_meta) + "\n"
            for key in ["stilling", "bransje"]:
                out += str(key).upper() + ":\n"
                try:
                    sorted_values = sorted(list(zip(data_dict[key].keys(), data_dict[key].values())),
                                           key=lambda x: float(x[1]), reverse=True)
                except KeyError as e:
                    if num_meta == 0:
                        logger.error("No metadata")

                for val in sorted_values:
                    out += str(val[0]) + ";" + str(val[1]) + "\n"
            out += "\n \n \n"
        handle.write(out.encode("utf8"))




    #     for resource_key in user_dict.keys():
    #         if resource_key[1] ==1:
    #             if resource_key[0] in emne_to_fagomrade_navn.keys():
    #                 print(emne_to_fagomrade_navn[resource_key[0]])




# pickle.dump(cluster_dict_with_id, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\users\clusters_with_id.pickle","wb"))

#


# connections_path = r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\connections-April2015.pickle"
# key_path = r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\users\keys.pickle"

# conns = pickle.load(open(connections_path,"rb"))
# keys = pickle.load(open(key_path, "rb"))
# clusters = pickle.load(open(cluster_path, "rb"))
# labels = []
#
# cluster_dict_by_uid = dict()
# for cluster_key in clusters.keys():
#     for user in clusters[cluster_key]:
#         cluster_dict_by_uid[user] = cluster_key
# for i in range(len(keys)):
#     key = keys[i][0]
#     labels.append(cluster_dict_by_uid.get(key,-1))
#
#


# adj= lgn.adjacency(conns,labels)
# adj.open()
# print(conns)
# score = silhouette_score(conns, labels, metric="precomputed")
# print(score)


# create_clustering_year(connections_path,key_path, cluster_path)


# write_trending_categories_for_clusters_month('June2016')
# write_trending_categories_for_clusters_year(cluster_path,data_path)
# # # #
# hit_dict = pickle.load(
#     open(r"C:\Users\Student\PycharmProjects\masteroppgave\data\hit_dict_index_on_user_id.pickle", "rb"))


#
# # find_most_common_topics_by_cluster(clusters_with_id, hit_dict, algorithm="louvain_src")





# # cluster_dict = dict()
# # for i in range(len(labels)):
# #     if labels[i] in cluster_dict.keys():
# #         cluster_dict[labels[i]].append(i)
# #     else:
# #        cluster_dict[labels[i]] = [i]
# # #
# # pickle.dump(cluster_dict, open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\louvain_cluster_dict_1000_samples.pickle","wb"))
# # cluster_dict = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\louvain_cluster_dict_1000_samples.pickle","rb"))
# # print(cluster_dict)
# # print(cluster_dict.keys8)
#
# # #

# #
# #
# # #
# occurs = dict()
# for value in labels:
#     if value in occurs.keys():
#         occurs[value] +=1
#     else:
#         occurs[value] = 1
# print (occurs)
#
# #
# #
# # del_keys = []
# # for key in occurs.keys():
# #     if occurs[key] < 5:
# #         del_keys.append(key)
# #
# # for key in del_keys:
# #     occurs.pop(key)
# # print(occurs)
# #
#
#
# #
# # #
# # # # # #

# #
# #



#
# for i in range(1,40):
#     conns = []
#     for row in similarities:
#         row = sorted(list(row), reverse =True )
#         conns.append(row[i])
#     plt.plot(sorted(conns, reverse = True))
#     plt.title(i)
#     plt.show()



# for key in cluster_dict:
#     print(key, len(list(cluster_dict[key])))


# G = nx.from_numpy_matrix(similarities)
# dendrogram = community.generate_dendrogram(G)
# print(len(dendrogram))
# size = float(len(set(partition.values())))
# pos = nx.spring_layout(G)
# count = 0.
# for com in set(partition.values()):
#     count += 1
#     list_nodes = [nodes for nodes in partition.keys() if partition[nodes] == com]
#     nx.draw_networkx_nodes(G, pos, list_nodes, node_size=20, node_color=str(count/size))
#
# nx.draw_networkx_edges(G, pos, alpha=0.5)
# plt.show()
#
#
# #
# clusters = pickle.load(open(r"C:\Users\Student\PycharmProjects\masteroppgave\results\fwpc\months\clusters-June2016.pickle","rb"))
# with open(r"clusters.csv","w") as handle:
#     for key, val in clusters.items():
#         for id in val:
#             handle.write(str(id)+";" +str(key)+ "\n")





# with open(r"C:\Users\Student\PycharmProjects\masteroppgave\statistics.txt", "a") as handle:
#     handle.write("clusters for FWPC created in"+ str(datetime.timedelta(seconds=time.time() - prog_start)))

print("total time: ", str(datetime.timedelta(seconds=time.time() - prog_start)))